"""
	NAME:		UserSniffer
	DESCRIPTION:	Returns client info based upon REMOTE_USER_AGENT
	VERSION:	1.1
	AUTHOR:		Seamus Venasse (Seamus.Venasse@polaris.ca)
	CREATED:	Thursday, August 24, 2000

	Based upon PHPClientSniffer by Roger Raymond
	http://phpclasses.upperdesign.com/browse.html/package/81

	DOCUMENTATION:
		- Install into the Zope "Extensions" directory
		- Add a new "External Method" to your Zope web folder
		  ID: UserSniffer
		  Function Name: UserSniffer
		  Python Module: UserSniffer
		- In your DTML Document, place a call to UserSniffer as in the
		  example below

	EXAMPLE USAGE:
		<dtml-call "REQUEST.set( 'sniffer', UserSniffer() )">
		<dtml-if "sniffer[ 'browser' ] == 'IE'">
			You're using Internet Explorer Version <dtml-var "sniffer[ 'version' ]">
		<dtml-else>
			You're not using IE, but your browser version is <dtml-var "sniffer[ 'version' ]">
		</dtml-if>

"""
	

import string
import re

def UserSniffer( self ):
	sniffer = {}
	sniffer[ 'browser' ] = ''
	sniffer[ 'version' ] = 0
	sniffer[ 'majorver' ] = 0
	sniffer[ 'minorver' ] = 0
	sniffer[ 'aol' ] = 0
	sniffer[ 'webtv' ] = 0
	sniffer[ 'javascript' ] = 0.0
	sniffer[ 'platform' ] = 'Unknown'
	sniffer[ 'os' ] = 'Unknown'
	sniffer[ 'ip' ] = 'Unknown'

	UA = self.REQUEST.HTTP_USER_AGENT


	# Determine Name and Version

	iever1 = re.search( 'MSIE ([0-9].[0-9a-zA-Z]{1,4})', UA, re.IGNORECASE )
	iever2 = re.search( 'Microsoft Internet Explorer ([0-9].[0-9a-zA-Z]{1,4})', UA, re.IGNORECASE )
	opver1 = re.search( 'Opera ([0-9].[0-9a-zA-Z]{1,4})', UA, re.IGNORECASE )
	opver2 = re.search( 'Opera/([0-9].[0-9a-zA-Z]{1,4})', UA, re.IGNORECASE )
	icver1 = re.search( 'iCab ([0-9].[0-9a-zA-Z]{1,4})', UA, re.IGNORECASE )
	icver2 = re.search( 'iCab/([0-9].[0-9a-zA-Z]{1,4})', UA, re.IGNORECASE )
	nsver1 = re.search( 'Netscape6/([0-9].[0-9a-zA-Z]{1,4})', UA, re.IGNORECASE )
	nsver2 = re.search( 'Mozilla/([0-9].[0-9a-zA-Z]{1,4})', UA, re.IGNORECASE )

	if ( iever1 ):
		sniffer[ 'version' ] = iever1.group( 1 )
		sniffer[ 'browser' ] = 'IE'
	elif ( iever2 ):
		sniffer[ 'version' ] = iever2.group( 1 )
		sniffer[ 'browser' ] = 'IE'
	elif ( opver1 ):
		sniffer[ 'version' ] = opver1.group( 1 )
		sniffer[ 'browser' ] = 'Opera'
	elif ( opver2 ):
		sniffer[ 'version' ] = opver2.group( 1 )
		sniffer[ 'browser' ] = 'Opera'
	elif ( icver1 ):
		sniffer[ 'version' ] = icver1.group( 1 )
		sniffer[ 'browser' ] = 'iCab'
	elif ( icver2 ):
		sniffer[ 'version' ] = icver2.group( 1 )
		sniffer[ 'browser' ] = 'iCab'
	elif ( nsver1 ):
		sniffer[ 'version' ] = nsver1.group( 1 )
		sniffer[ 'browser' ] = 'Netscape'
	elif ( nsver2 ):
		sniffer[ 'version' ] = nsver2.group( 1 )
		sniffer[ 'browser' ] = 'Netscape'
	else:
		sniffer[ 'version' ] = 0
		sniffer[ 'browser' ] = 'Unknown'

	sniffer[ 'version' ] = float( sniffer[ 'version' ] )


	# Determine Major and Minor Version

	if ( sniffer[ 'version' ] > 0 ):
		try:
			string.index( str( sniffer[ 'version' ] ), '.' )
			tmp = string.split( str( sniffer[ 'version' ] ), '.' )
			sniffer[ 'majorver' ] = int( tmp[ 0 ] )
			sniffer[ 'minorver' ] = float( '0.' + tmp[ 1 ] )
		except:
			sniffer[ 'majorver' ] = sniffer[ 'minorver' ]


	# Determine if AOL or WEBTV

	if ( re.search( 'aol', UA, re.IGNORECASE ) ):
		sniffer[ 'aol' ] = 1

	elif ( re.search( 'webtv', UA, re.IGNORECASE ) ):
		sniffer[ 'webtv' ] = 1


	# Determine Platform and OS

	if ( ( re.search( 'win16', UA, re.IGNORECASE ) != None ) | ( re.search( 'windows 3.1', UA, re.IGNORECASE ) != None ) | 
	     ( re.search( 'windows 16-bit', UA, re.IGNORECASE ) != None ) | ( re.search( '16bit', UA, re.IGNORECASE ) != None ) ):
		sniffer[ 'platform' ] = 'Win16'
		sniffer[ 'os' ] = 'Win31'

	if ( ( re.search( 'win95', UA, re.IGNORECASE ) != None ) | ( re.search( 'windows 95', UA, re.IGNORECASE ) != None ) ):
		sniffer[ 'platform' ] = 'Win32'
		sniffer[ 'os' ] = 'Win95'

	elif ( ( re.search( 'win98', UA, re.IGNORECASE ) != None ) | ( re.search( 'windows 98', UA, re.IGNORECASE ) != None ) ):
		sniffer[ 'platform' ] = 'Win32'
		sniffer[ 'os' ] = 'Win98'

	elif ( ( re.search( 'winnt', UA, re.IGNORECASE ) != None ) | ( re.search( 'windows nt', UA, re.IGNORECASE ) != None ) ):
		sniffer[ 'platform' ] = 'Win32'
		sniffer[ 'os' ] = 'WinNT'

	else:
		sniffer[ 'platform' ] = 'Win32'
		sniffer[ 'os' ] = 'Win9xNT'


	# OS/2
	if ( re.search( 'OS/2', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'OS2'
		sniffer[ 'os' ] = 'OS2'

	if ( re.search( 'IBM-WEBEXPLORER', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'OS2'
		sniffer[ 'os' ] = 'OS2'

	# Mac 68000
	if ( re.search( '68k', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Mac'
		sniffer[ 'os' ] = 'Mac68k'

	if ( re.search( '68000', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Mac'
		sniffer[ 'os' ] = 'Mac68k'

	# Mac PowerPC
	if ( re.search( 'ppc', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Mac'
		sniffer[ 'os' ] = 'MacPPC'

	if ( re.search( 'powerpc', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Mac'
		sniffer[ 'os' ] = 'MacPPC'

	# Check for UNIX Flavour

	# SunOS
	if ( re.search( 'sunos', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'sun'

	if ( re.search( 'sunos 4', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'sun4'

	elif ( re.search( 'sunos 5', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'sun5'

	elif ( re.search( 'i86', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'suni86'

	# Irix
	if ( re.search( 'irix', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'irix'

	if ( re.search( 'irix 5', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'irix5'

	elif ( re.search( 'irix 6', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'irix6'

	# HP-UX
	if ( re.search( 'hp-ux', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'hpux'

	if ( ( re.search( 'hp-ux', UA, re.IGNORECASE ) != None ) & ( re.search( '09.', UA, re.IGNORECASE ) != None ) ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'hpux9'

	elif ( ( re.search( 'hp-ux', UA, re.IGNORECASE ) != None ) & ( re.search( '10.', UA, re.IGNORECASE ) != None ) ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'hpux10'

	# AIX
	if ( re.search( 'aix', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'aix'

	if ( re.search( 'aix1', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'aix1'

	elif ( re.search( 'aix2', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'aix2'

	elif ( re.search( 'aix3', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'aix3'

	elif ( re.search( 'aix4', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'aix4'

	# Linux
	if ( re.search( 'inux', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'linux'

	# Unixware
	if ( re.search( 'unix_system_v', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'unixware'

	# mpras
	if ( re.search( 'ncr', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'mpras'

	# Reliant
	if ( re.search( 'reliantunix', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'reliant'

	# DEC
	if ( re.search( 'dec', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'dec'

	if ( re.search( 'osfl', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'dec'

	if ( re.search( 'alphaserver', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'dec'

	if ( re.search( 'ultrix', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'dec'

	if ( re.search( 'alphastation', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'dec'

	# Sinix
	if ( re.search( 'sinix', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'sinix'

	# BSD
	if ( re.search( 'bsd', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'bsd'

	# FreeBSD
	if ( re.search( 'freebsd', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'freebsd'

	# VMS
	if ( re.search( 'vax', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'vms'

	if ( re.search( 'openvms', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'vms'

	# SCO
	if ( re.search( 'sco', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'sco'

	if ( re.search( 'unix_sv', UA, re.IGNORECASE ) != None ):
		sniffer[ 'platform' ] = 'Unix'
		sniffer[ 'os' ] = 'sco'



	# Assume JavaScript Version

	ie = ( sniffer[ 'browser' ] == 'IE' )
	ie5 = ( ie & ( sniffer[ 'majorver' ] >= 5 ) )
	ie4 = ( ie & ( sniffer[ 'majorver' ] == 4 ) )
	ie3 = ( ie & ( sniffer[ 'majorver' ] == 3 ) )

	ns = ( sniffer[ 'browser' ] == 'Netscape' )
	ns5 = ( ns & ( sniffer[ 'majorver' ] >= 5 ) )
	ns4 = ( ns & ( sniffer[ 'majorver' ] == 4 ) )
	ns3 = ( ns & ( sniffer[ 'majorver' ] == 3 ) )
	ns2 = ( ns & ( sniffer[ 'majorver' ] == 2 ) )

	opera = ( sniffer[ 'browser' ] == 'Opera' )

	if ( ns5 ):
		sniffer[ 'javascript' ] = 1.4
	if ( ie5 ):
		sniffer[ 'javascript' ] = 1.3
	if ( ( ns4 & ( sniffer[ 'minorver' ] > 0.05 ) ) | ie4 ):
		sniffer[ 'javascript' ] = 1.3
	if ( ns4 & ( sniffer[ 'minorver' ] <= 0.05 ) ):
		sniffer[ 'javascript' ] = 1.2
	if ( ns3 | opera ):
		sniffer[ 'javascript' ] = 1.1
	if ( ns2 | ie3 ):
		sniffer[ 'javascript' ] = 1.0


	# Grab IP Address

	sniffer[ 'ip' ] = self.REQUEST.REMOTE_ADDR

	return sniffer
